/*  File:     cnctInfoSocketClient.java 
    Project:  OOPracktikum 2001
	            Connect-Game Info Client
	            Patrick Scheibe, Markus Kreuz, Georg Martius
    Package:  cnctInfoSocket
    Autor:    
    Descript: Klasse um das Protokoll zu verwalten:
    last mod: 
    date
    Mon Jun  4 00:28:47 CEST 2001
*/

package cnctInfoSocket;

import point;
import cnctParser.*;

public class cnctInfoSocketClient extends lineSocket {
    // Eventhandler-objekt
    private cnctInfoSocketEvent event;
    private boolean stopListening;
    private boolean firstread;
    private int iHaveSend;
    
    public final static int NOTHING=0;
    public final static int LOGIN=1;
    
    
    private cnctInfoPlayerList playerList;
    private cnctInfoGameList gameList;
   
    private String[] players;
    private int[] games;    
    private int pollplayers;
    private int pollgames;

    private boolean canISend;           // legt fest ob man senden kann, da man schon eine Anwort auf die vorhergehende Frage bekam
    
    /////////////////////////////////////////// F U N K T I O N E N ///////////////////////////
    // cnctInfoSocketClient(cnctInfoSocketEvent _event, String _hostname, int _port);
    // init(String _hostname, int _port);
    // public void run();
    // public void stop();
    // public void close();
    // private void handleInput(String input);
    //###schneidet den Kommentar ab    
    // private String cutComment(String input);
    //###extrahiert den Kommentar
    // private String getComment(String input);
    //###was habe ich zuletzt gesendet siehe Konstanten
    // public int getWhatISend(){
    //###meldet sich an    
    // public boolean login(String name);
    //###meldet sich ab, geht nur wenn idle, waiting
    // public boolean logout();
    //###Tritt einem Spiel bei
    // public boolean join(int gameNr);
    //###Fragt nach Spielern
    // public boolean who();
    //###Eroeffnet neues Spiel
    // public boolean newGame(int gamesize, int color);

    /////////////////////////////////////////// I M P L E M E N T I E R U N G ///////////////////////////

    
    //Konstruktor fuer Initialisierung mit Werten
    //--------------------------------------------------       
    public cnctInfoSocketClient(cnctInfoSocketEvent _event, String _hostname, int _port) throws cnctSocketException{
       super(_hostname, _port);
       event=_event;
       stopListening=false;
       canISend=true;
       pollplayers=-1;
       pollgames=-1;
       firstread=true;
       iHaveSend=NOTHING;
    }
    

    public int getWhatISend(){
      return iHaveSend;
    }
    
    /*lauscht auf Socket und ruft Events auf*/
    //---------------------------------------------------------------------------------
    public void run() {
      String buffer;
     
      while (stopListening==false){
        buffer=read();
        
        if (buffer!=null && buffer.length()>0) {
          try{
            if (iHaveSend!=NOTHING && !firstread){
                handleInput(buffer);
            }
            firstread=false;
          }catch(cnctSocketException e){
            e.printStackTrace(System.out);
          }
        }
           
        try {
          sleep(500);
        }catch(InterruptedException e) {
          //###
          System.out.println("cnctInfoSocketClient::run: error while sleep ->" + e);
        }      
      }
      super.close();
      
    }
    
    // behandelt den String, der auf dem Socket gelesen wurde
    //---------------------------------------------------------------------------------
    public void handleInput(String input) throws cnctSocketException{
      String buffer, comment;
      int submit, pos;
      Integer tempint;
      parser inputparser = new parser();
      knoten syntax;
      int i;
      
      System.out.println(">> " + input);
      
      // Kommentare aus dem String extrahieren
      comment = getComment(input);
      input = cutComment(input);
      
      // String parsen
      inputparser.baueBaum(input);

      canISend = true;
        
      syntax=inputparser.getSyntaxBaum();
      if(syntax.getTyp()!=knoten.KONSTRUKTOR){
        throw new cnctSocketException("Falscher Syntax bei Antwort von InfoServer!\nMuss mit Bezeichner beginnen: " + input);
      }
      
      if (syntax.compareName("OK")){
         event.okReceived();
      }else 
      if (syntax.compareName("Players")){  // So die Player kommen jetzt als Liste und mssen einzeln abgefragt werden
         if (playerList!=null) playerList.removeAllElements();
         playerList = new cnctInfoPlayerList();
         if(syntax.getKind(0)== null || syntax.getKind(0).getTyp() != knoten.LISTE)  throw new cnctSocketException("SpielerListe ist keine Liste");  
         players = new String[syntax.getKind(0).getKinderZahl()];
         for(i=0;i<syntax.getKind(0).getKinderZahl();i++)  // Alle Spieler speichern
         {
            players[i]= new String(syntax.getKind(0).getKind(i).getName());
         }
         pollplayers=0;
         getOnePlayer();
      }else
      if (syntax.compareName("Player")){  // ein Player kommt
         if(pollplayers>=0){
           playerList.analyseSyntaxAndAdd(syntax.getKind(0),players[pollplayers]);
           pollplayers++;
           if (pollplayers<players.length){
             if(!getOnePlayer()){
              pollplayers=-1;
              throw new cnctSocketException("Fehler beim Abfragen der Spieler");  
             }
           }else{ // Fertig! Alle Player bekommen
             pollplayers=-1;
             event.players(playerList);
			 //!!!
//			super.send("Games All");
           };
         }
      }else
      if(syntax.compareName("Games")){// So die Tische kommen jetzt als Liste und mssen einzeln abgefragt werden
         if (gameList!=null) gameList.removeAllElements();
         gameList = new cnctInfoGameList();
         if(syntax.getKind(0)== null || syntax.getKind(0).getTyp() != knoten.LISTE)  
		 	throw new cnctSocketException("TischListe ist keine Liste");
			  
         games = new int[syntax.getKind(0).getKinderZahl()];
 
        for(i=0;i<syntax.getKind(0).getKinderZahl();i++)  // Alle Tische speichern
        {
            games[i] = syntax.getKind(0).getKind(i).getZahl();
         }
 	 
         pollgames=0;
         getOneGame();
      }else  
	  if(syntax.compareName("Game")){  // ein Tisch kommt
         if(pollgames>=0){
           gameList.analyseSyntaxAndAdd(syntax.getKind(0),games[pollgames]);
           pollgames++;
           if (pollgames<games.length){
             if(!getOneGame()){
              pollgames=-1;
              throw new cnctSocketException("Fehler beim Abfragen der Tische");  
             }
           }else{ // Fertig! Alle Player bekommen
             pollplayers=-1;
             event.games(gameList);
           };
         }
      }else
      if(syntax.compareName("Error")){
        event.errorReceived();
      }else{
        System.out.println("Unbekannte Antwort von Server: " + input);
      }


    }
    // Fragt Server nach einem Player und schaut, ob es weitere Player gibt
    //  wenn zuende, dann wird pollplayers auf -1 gesetzt, so dass dann die 
    //  funktion HandleInput  den Event "players" ausloesen kann    
    //---------------------------------------------------------------------------------
    private boolean getOnePlayer(){
      String playername;
      if (players!=null && pollplayers!=-1){
        if(players.length > pollplayers){
          playername = players[pollplayers];
          if (playername!=null){
            super.send("Player \"" + playername + "\"");
            canISend=false;
          }
        }else{
          return false;
        }
      }
      return true;
    }   
     
	// Fragt Server nach einem Tisch und schaut, ob es weitere Player gibt
    //  wenn zuende, dann wird pollgames auf -1 gesetzt, so dass dann die 
    //  funktion HandleInput  den Event "games" ausloesen kann    
    //---------------------------------------------------------------------------------
    private boolean getOneGame(){
      int gamenumber;
      if (games!=null && pollgames!=-1){
        if(games.length > pollgames){
          gamenumber = games[pollgames];
          if (gamenumber!=0){
            super.send("Game " + gamenumber);
            canISend=false;
          }
        }else{
          return false;
        }
      }
      return true;
    }   
    //schneidet den Kommentar ab    
    //---------------------------------------------------------------------------------
    private boolean extractPlayer(knoten baum){
      return false;
    }
    
    //schneidet den Kommentar ab    
    //---------------------------------------------------------------------------------
    private String cutComment(String input){
       int routepos = input.indexOf("#");
       if (routepos<=0)
         return input;
       else
         return input.substring(0,routepos);
    }
    
    //extrahiert den Kommentar
    //---------------------------------------------------------------------------------
    private String getComment(String input){
       int routepos = input.indexOf("#");
       if (routepos<=0)
         return new String("");
       else
         return input.substring(routepos+1,input.length());
    }

    // Meldet sich an.
    // return false: kann nicht senden, da nicht bereit.
    //---------------------------------------------------------------------------------
    public boolean login(String name){
      if(canISend==true){
        canISend=false;
        iHaveSend=LOGIN;
        super.send("Login \"" + name + "\"");
        return true;
      }else{
        return false;
      }
    }

    // meldet sich ab, geht nur wenn idle, waiting
    // return false: kann nicht senden, da nicht bereit.
    //---------------------------------------------------------------------------------
    public boolean logout(){
      if(canISend==true){
        canISend=false;
        super.send("Logout");
        return true;
      }else{
        return false;
      }
    }

    
    // Tritt einem Spiel bei
    // return false: kann nicht senden, da nicht bereit.
    //---------------------------------------------------------------------------------
    public boolean join(int gameNr){
      if(canISend==true){
        canISend=false;
        super.send("Join { game = " + gameNr + " }");
        return true;
      }else{
        return false;
      }
    }

    // Tritt einem Spiel als Zuschauer bzw. Hirn bei
    // return false: kann nicht senden, da nicht bereit.
    //---------------------------------------------------------------------------------
    public boolean watch(int gameNr){
      if(canISend==true){
        canISend=false;
        //!!!!  FARBE
        super.send("Watch { game =" + gameNr + " , color = Schwarz}");
        return true;
      }else{
        return false;
      }
    }

    
    //Fragt nach Spielern
    // return false: kann nicht senden, da nicht bereit.
    //---------------------------------------------------------------------------------        
    public boolean who(){
     int i;
	 if(canISend==true){
        canISend=false;
        super.send("Players All");
        return true;
      }else{
        return false;
      }
    }

    //Eroeffnet neues Spiel    
    // return false: kann nicht senden, da nicht bereit.
    //---------------------------------------------------------------------------------        
    public boolean newGame(int gamesize, int color){
     if(canISend==true){
        canISend=false;
        String colorstr;
        if(color==0) colorstr=new String("Schwarz");
        else colorstr=new String("Weiss");
        
        super.send("New { color= " + colorstr + " ,  size = "+ gamesize + "}");
        return true;
      }else{
        return false;
      }
    }
    
    // schliet den Socket.
    //---------------------------------------------------------------------------------    
    public void close(){
      stopListening=true;
    }
      
}
